(*******************************************************************************
  Author
  ------
  Kai Rhr (Seth)
  email : info@exec-dev.de
  visit: http://www.exec-dev.de | http://www.delphigl.com

  License
  -------
  Copyright (C) 2007 www.delphigl.com

  This license applies to everything on www.delphigl.com, except where otherwise noted.

  This software is provided 'as-is', without any express or implied warranty.  In no event will the authors 
  be held liable for any damages arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose, including commercial applications, 
  and to alter it and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not claim that you wrote the original 
  software. If you use this software in a product, an acknowledgment in the product documentation would be 
  appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Description
  -----------
  Demonstration of the separating axis theorem (collision of two convex polygons)    

  Beschreibung
  -------------  
  Demonstration des Separating Axis Theorems (Kollision zweier konvexer Polygone)   

*******************************************************************************)

unit UMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Vectors, Collision, ExtCtrls, StdCtrls;

type
  TMainForm = class(TForm)
    Image1: TImage;
    Label1: TLabel;
    procedure Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    A, B: TPolygon;
    procedure DrawPolygon(A: TPolygon);
    procedure DrawScene;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

procedure TMainForm.DrawPolygon(A: TPolygon);
var
  i, l: integer;
begin
  for i := 0 to A.Count - 1 do
  begin
    l := i + 1;
    if l > (A.Count - 1) then
      l := 0;
    Image1.Canvas.MoveTo(round(A.vertices_abs[l].x), round(A.vertices_abs[l].y));
    Image1.Canvas.LineTo(round(A.vertices_abs[i].x), round(A.vertices_abs[i].y));
  end;
end;

procedure TMainForm.DrawScene;
begin
  image1.Picture.Bitmap := nil;
  image1.Canvas.Pen.color := clblack;
  if PolyPolyIntersect(a, b) then
    image1.Canvas.Pen.color := clred;
  DrawPolygon(A);
  DrawPolygon(B);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  doublebuffered := true;
  A := TPolygon.Create;
  B := TPolygon.Create;
  with A do
  begin
    position := to_v2f(200, 200);
    AddVertex(to_v2f(50, -50));
    AddVertex(to_v2f(50, 50));
    AddVertex(to_v2f(-50, 50));
    AddVertex(to_v2f(-50, -50));
  end;
  with B do
  begin
    position := to_v2f(250, 250);
    AddVertex(to_v2f(50, -50));
    AddVertex(to_v2f(25, 25));
    AddVertex(to_v2f(-50, -50));
  end;
  DrawScene;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  A.Free;
  B.Free;
end;

procedure TMainForm.Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if ssleft in shift then
  begin
    B.position := to_v2f(x, y);
    DrawScene;
  end;  
end;

end.
